@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2010  Université de Bordeaux 1
- * Copyright (C) 2010  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2011  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -28,8 +28,10 @@ void cpu_codelet(void *descr[], void *_args)
         float *multiplier = (float *)_args;
         unsigned i, j, k;
 
-        for(k=0; k<nz ; k++) {
-                for(j=0; j<ny ; j++) {
+        for(k=0; k<nz ; k++)
+	{
+                for(j=0; j<ny ; j++)
+		{
                         for(i=0; i<nx ; i++)
                                 block[(k*ldz)+(j*ldy)+i] *= *multiplier;
                 }
