@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2009, 2010-2011  Université de Bordeaux 1
- * Copyright (C) 2010  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2011  Télécom-SudParis
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,8 +19,8 @@
 #include "lu_kernels_model.h"
 
 /*
- * As a convention, in that file, descr[0] is represented by A,
- * 				  descr[1] is B ...
+ * As a convention, in that file, buffers[0] is represented by A,
+ * 				  buffers[1] is B ...
  */
 
 /*
@@ -41,22 +42,22 @@
  *
  */
 
-double task_11_cost(starpu_buffer_descr *descr)
+double task_11_cost(struct starpu_task *task, unsigned nimpl)
 {
 	uint32_t n;
 
-	n = starpu_matrix_get_nx(descr[0].handle);
+	n = starpu_matrix_get_nx(task->handles[0]);
 
 	double cost = ((n*n*n)/537.5);
 
 	return PERTURBATE(cost);
 }
 
-double task_12_cost(starpu_buffer_descr *descr)
+double task_12_cost(struct starpu_task *task, unsigned nimpl)
 {
 	uint32_t n;
 
-	n = starpu_matrix_get_nx(descr[0].handle);
+	n = starpu_matrix_get_nx(task->handles[0]);
 
 /*	double cost = ((n*n*n)/1744.695); */
 	double cost = ((n*n*n)/3210.80);
@@ -66,11 +67,11 @@ double task_12_cost(starpu_buffer_descr *descr)
 }
 
 
-double task_21_cost(starpu_buffer_descr *descr)
+double task_21_cost(struct starpu_task *task, unsigned nimpl)
 {
 	uint32_t n;
 
-	n = starpu_matrix_get_nx(descr[0].handle);
+	n = starpu_matrix_get_nx(task->handles[0]);
 
 /*	double cost = ((n*n*n)/1744.695); */
 	double cost = ((n*n*n)/3691.53);
@@ -81,13 +82,13 @@ double task_21_cost(starpu_buffer_descr *descr)
 
 
 
-double task_22_cost(starpu_buffer_descr *descr)
+double task_22_cost(struct starpu_task *task, unsigned nimpl)
 {
 	uint32_t nx, ny, nz;
 
-	nx = starpu_matrix_get_nx(descr[2].handle);
-	ny = starpu_matrix_get_ny(descr[2].handle);
-	nz = starpu_matrix_get_ny(descr[0].handle);
+	nx = starpu_matrix_get_nx(task->handles[2]);
+	ny = starpu_matrix_get_ny(task->handles[2]);
+	nz = starpu_matrix_get_ny(task->handles[0]);
 
 	double cost = ((nx*ny*nz)/4110.0);
 
@@ -101,11 +102,11 @@ double task_22_cost(starpu_buffer_descr *descr)
  */
 
 
-double task_11_cost_cuda(starpu_buffer_descr *descr)
+double task_11_cost_cuda(struct starpu_task *task, enum starpu_perf_archtype arch, unsigned nimpl)
 {
 	uint32_t n;
 
-	n = starpu_matrix_get_nx(descr[0].handle);
+	n = starpu_matrix_get_nx(task->handles[0]);
 
 	double cost = ((n*n*n)/1853.7806);
 
@@ -113,11 +114,11 @@ double task_11_cost_cuda(starpu_buffer_descr *descr)
 	return PERTURBATE(cost);
 }
 
-double task_12_cost_cuda(starpu_buffer_descr *descr)
+double task_12_cost_cuda(struct starpu_task *task, enum starpu_perf_archtype arch, unsigned nimpl)
 {
 	uint32_t n;
 
-	n = starpu_matrix_get_nx(descr[0].handle);
+	n = starpu_matrix_get_nx(task->handles[0]);
 
 	double cost = ((n*n*n)/42838.5718);
 
@@ -126,11 +127,11 @@ double task_12_cost_cuda(starpu_buffer_descr *descr)
 }
 
 
-double task_21_cost_cuda(starpu_buffer_descr *descr)
+double task_21_cost_cuda(struct starpu_task *task, enum starpu_perf_archtype arch, unsigned nimpl)
 {
 	uint32_t n;
 
-	n = starpu_matrix_get_nx(descr[0].handle);
+	n = starpu_matrix_get_nx(task->handles[0]);
 
 	double cost = ((n*n*n)/49208.667);
 
@@ -140,13 +141,13 @@ double task_21_cost_cuda(starpu_buffer_descr *descr)
 
 
 
-double task_22_cost_cuda(starpu_buffer_descr *descr)
+double task_22_cost_cuda(struct starpu_task *task, enum starpu_perf_archtype arch, unsigned nimpl)
 {
 	uint32_t nx, ny, nz;
 
-	nx = starpu_matrix_get_nx(descr[2].handle);
-	ny = starpu_matrix_get_ny(descr[2].handle);
-	nz = starpu_matrix_get_ny(descr[0].handle);
+	nx = starpu_matrix_get_nx(task->handles[2]);
+	ny = starpu_matrix_get_ny(task->handles[2]);
+	nz = starpu_matrix_get_ny(task->handles[0]);
 
 	double cost = ((nx*ny*nz)/57523.560);
 
@@ -160,11 +161,11 @@ double task_22_cost_cuda(starpu_buffer_descr *descr)
  *
  */
 
-double task_11_cost_cpu(starpu_buffer_descr *descr)
+double task_11_cost_cpu(struct starpu_task *task, enum starpu_perf_archtype arch, unsigned nimpl)
 {
 	uint32_t n;
 
-	n = starpu_matrix_get_nx(descr[0].handle);
+	n = starpu_matrix_get_nx(task->handles[0]);
 
 	double cost = ((n*n*n)/537.5);
 
@@ -172,11 +173,11 @@ double task_11_cost_cpu(starpu_buffer_descr *descr)
 	return PERTURBATE(cost);
 }
 
-double task_12_cost_cpu(starpu_buffer_descr *descr)
+double task_12_cost_cpu(struct starpu_task *task, enum starpu_perf_archtype arch, unsigned nimpl)
 {
 	uint32_t n;
 
-	n = starpu_matrix_get_nx(descr[0].handle);
+	n = starpu_matrix_get_nx(task->handles[0]);
 
 	double cost = ((n*n*n)/6668.224);
 
@@ -185,11 +186,11 @@ double task_12_cost_cpu(starpu_buffer_descr *descr)
 }
 
 
-double task_21_cost_cpu(starpu_buffer_descr *descr)
+double task_21_cost_cpu(struct starpu_task *task, enum starpu_perf_archtype arch, unsigned nimpl)
 {
 	uint32_t n;
 
-	n = starpu_matrix_get_nx(descr[0].handle);
+	n = starpu_matrix_get_nx(task->handles[0]);
 
 	double cost = ((n*n*n)/6793.8423);
 
@@ -199,13 +200,13 @@ double task_21_cost_cpu(starpu_buffer_descr *descr)
 
 
 
-double task_22_cost_cpu(starpu_buffer_descr *descr)
+double task_22_cost_cpu(struct starpu_task *task, enum starpu_perf_archtype arch, unsigned nimpl)
 {
 	uint32_t nx, ny, nz;
 
-	nx = starpu_matrix_get_nx(descr[2].handle);
-	ny = starpu_matrix_get_ny(descr[2].handle);
-	nz = starpu_matrix_get_ny(descr[0].handle);
+	nx = starpu_matrix_get_nx(task->handles[2]);
+	ny = starpu_matrix_get_ny(task->handles[2]);
+	nz = starpu_matrix_get_ny(task->handles[0]);
 
 	double cost = ((nx*ny*nz)/4203.0175);
 
@@ -213,11 +214,13 @@ double task_22_cost_cpu(starpu_buffer_descr *descr)
 	return PERTURBATE(cost);
 }
 
-struct starpu_perfmodel_t model_11 = {
-	.cost_model = task_11_cost,
-	.per_arch = { 
-		[STARPU_CPU_DEFAULT] = { .cost_model = task_11_cost_cpu },
-		[STARPU_CUDA_DEFAULT] = { .cost_model = task_11_cost_cuda }
+struct starpu_perfmodel model_11 =
+{
+	.cost_function = task_11_cost,
+	.per_arch =
+	{
+		[STARPU_CPU_DEFAULT][0] = { .cost_function = task_11_cost_cpu },
+		[STARPU_CUDA_DEFAULT][0] = { .cost_function = task_11_cost_cuda }
 	},
 	.type = STARPU_HISTORY_BASED,
 #ifdef STARPU_ATLAS
@@ -229,11 +232,13 @@ struct starpu_perfmodel_t model_11 = {
 #endif
 };
 
-struct starpu_perfmodel_t model_12 = {
-	.cost_model = task_12_cost,
-	.per_arch = { 
-		[STARPU_CPU_DEFAULT] = { .cost_model = task_12_cost_cpu },
-		[STARPU_CUDA_DEFAULT] = { .cost_model = task_12_cost_cuda }
+struct starpu_perfmodel model_12 =
+{
+	.cost_function = task_12_cost,
+	.per_arch =
+	{
+		[STARPU_CPU_DEFAULT][0] = { .cost_function = task_12_cost_cpu },
+		[STARPU_CUDA_DEFAULT][0] = { .cost_function = task_12_cost_cuda }
 	},
 	.type = STARPU_HISTORY_BASED,
 #ifdef STARPU_ATLAS
@@ -245,11 +250,13 @@ struct starpu_perfmodel_t model_12 = {
 #endif
 };
 
-struct starpu_perfmodel_t model_21 = {
-	.cost_model = task_21_cost,
-	.per_arch = { 
-		[STARPU_CPU_DEFAULT] = { .cost_model = task_21_cost_cpu },
-		[STARPU_CUDA_DEFAULT] = { .cost_model = task_21_cost_cuda }
+struct starpu_perfmodel model_21 =
+{
+	.cost_function = task_21_cost,
+	.per_arch =
+	{
+		[STARPU_CPU_DEFAULT][0] = { .cost_function = task_21_cost_cpu },
+		[STARPU_CUDA_DEFAULT][0] = { .cost_function = task_21_cost_cuda }
 	},
 	.type = STARPU_HISTORY_BASED,
 #ifdef STARPU_ATLAS
@@ -261,11 +268,13 @@ struct starpu_perfmodel_t model_21 = {
 #endif
 };
 
-struct starpu_perfmodel_t model_22 = {
-	.cost_model = task_22_cost,
-	.per_arch = { 
-		[STARPU_CPU_DEFAULT] = { .cost_model = task_22_cost_cpu },
-		[STARPU_CUDA_DEFAULT] = { .cost_model = task_22_cost_cuda }
+struct starpu_perfmodel model_22 =
+{
+	.cost_function = task_22_cost,
+	.per_arch =
+	{
+		[STARPU_CPU_DEFAULT][0] = { .cost_function = task_22_cost_cpu },
+		[STARPU_CUDA_DEFAULT][0] = { .cost_function = task_22_cost_cuda }
 	},
 	.type = STARPU_HISTORY_BASED,
 #ifdef STARPU_ATLAS
