@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010-2011  Université de Bordeaux 1
- * Copyright (C) 2010, 2011  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009-2012  Université de Bordeaux 1
+ * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -46,41 +46,51 @@ TYPE **A_blocks;
 static void parse_args(int argc, char **argv)
 {
 	int i;
-	for (i = 1; i < argc; i++) {
-		if (strcmp(argv[i], "-size") == 0) {
+	for (i = 1; i < argc; i++)
+	{
+		if (strcmp(argv[i], "-size") == 0)
+		{
 			char *argptr;
 			size = strtol(argv[++i], &argptr, 10);
 		}
 
-		if (strcmp(argv[i], "-nblocks") == 0) {
+		if (strcmp(argv[i], "-nblocks") == 0)
+		{
 			char *argptr;
 			nblocks = strtol(argv[++i], &argptr, 10);
 		}
 
-		if (strcmp(argv[i], "-check") == 0) {
+		if (strcmp(argv[i], "-check") == 0)
+		{
 			check = 1;
 		}
 
-		if (strcmp(argv[i], "-piv") == 0) {
+		if (strcmp(argv[i], "-piv") == 0)
+		{
 			pivot = 1;
 		}
 
-		if (strcmp(argv[i], "-no-stride") == 0) {
+		if (strcmp(argv[i], "-no-stride") == 0)
+		{
 			no_stride = 1;
 		}
 
-		if (strcmp(argv[i], "-profile") == 0) {
+		if (strcmp(argv[i], "-profile") == 0)
+		{
 			profile = 1;
 		}
 
-		if (strcmp(argv[i], "-bound") == 0) {
+		if (strcmp(argv[i], "-bound") == 0)
+		{
 			bound = 1;
 		}
-		if (strcmp(argv[i], "-bounddeps") == 0) {
+		if (strcmp(argv[i], "-bounddeps") == 0)
+		{
 			bound = 1;
 			bounddeps = 1;
 		}
-		if (strcmp(argv[i], "-bounddepsprio") == 0) {
+		if (strcmp(argv[i], "-bounddepsprio") == 0)
+		{
 			bound = 1;
 			bounddeps = 1;
 			boundprio = 1;
@@ -122,7 +132,7 @@ void copy_blocks_into_matrix(void)
 				A_blocks[bi+nblocks*bj][i + j * blocksize];
 		}
 
-		/* free(A_blocks[bi+nblocks*bj]); */
+		starpu_free(A_blocks[bi+nblocks*bj]);
 	}
 }
 
@@ -164,6 +174,10 @@ static void init_matrix(void)
 		for (i = 0; i < size; i++)
 		{
 			A[i + j*size] = (TYPE)starpu_drand48();
+#ifdef COMPLEX_LU
+			/* also randomize the imaginary component for complex number cases */
+			A[i + j*size] += (TYPE)(I*starpu_drand48());
+#endif
 		}
 	}
 
@@ -248,12 +262,21 @@ static void check_result(void)
 	/* compute "LU - A" in L*/
 	CPU_AXPY(size*size, -1.0, A_saved, 1, L, 1);
 	display_matrix(L, size, size, "Residuals");
-	
+
+#ifdef COMPLEX_LU
+	double err = CPU_ASUM(size*size, L, 1);
+	int max = CPU_IAMAX(size*size, L, 1);
+	TYPE l_max = L[max];
+
+	FPRINTF(stderr, "Avg error : %e\n", err/(size*size));
+	FPRINTF(stderr, "Max error : %e\n", sqrt(creal(l_max)*creal(l_max)+cimag(l_max)*cimag(l_max)));
+#else
 	TYPE err = CPU_ASUM(size*size, L, 1);
 	int max = CPU_IAMAX(size*size, L, 1);
 
 	FPRINTF(stderr, "Avg error : %e\n", err/(size*size));
 	FPRINTF(stderr, "Max error : %e\n", L[max]);
+#endif
 
 	double residual = frobenius_norm(L, size);
 	double matnorm = frobenius_norm(A_saved, size);
@@ -262,19 +285,33 @@ static void check_result(void)
 
 	if (residual/(matnorm*size) > 1e-5)
 		exit(-1);
+
+	free(L);
+	free(U);
+	free(A_saved);
 }
 
 int main(int argc, char **argv)
 {
+	int ret;
+
 	parse_args(argc, argv);
 
-	starpu_init(NULL);
+#ifdef STARPU_SLOW_MACHINE
+	size /= 4;
+	nblocks /= 4;
+#endif
+
+	ret = starpu_init(NULL);
+	if (ret == -ENODEV)
+		return 77;
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	starpu_helper_cublas_init();
 
 	init_matrix();
 
-	unsigned *ipiv;
+	unsigned *ipiv = NULL;
 	if (check)
 		save_matrix();
 
@@ -296,24 +333,24 @@ int main(int argc, char **argv)
 			A_blocks = malloc(nblocks*nblocks*sizeof(TYPE **));
 			copy_matrix_into_blocks();
 
-			STARPU_LU(lu_decomposition_pivot_no_stride)(A_blocks, ipiv, size, size, nblocks);
+			ret = STARPU_LU(lu_decomposition_pivot_no_stride)(A_blocks, ipiv, size, size, nblocks);
 
 			copy_blocks_into_matrix();
 			free(A_blocks);
 		}
-		else 
+		else
 		{
 			struct timeval start;
 			struct timeval end;
 
 			gettimeofday(&start, NULL);
 
-			STARPU_LU(lu_decomposition_pivot)(A, ipiv, size, size, nblocks);
-	
+			ret = STARPU_LU(lu_decomposition_pivot)(A, ipiv, size, size, nblocks);
+
 			gettimeofday(&end, NULL);
 
 			double timing = (double)((end.tv_sec - start.tv_sec)*1000000 + (end.tv_usec - start.tv_usec));
-			
+
 			unsigned n = size;
 			double flop = (2.0f*n*n*n)/3.0f;
 			FPRINTF(stderr, "Synthetic GFlops (TOTAL) : \n");
@@ -322,40 +359,61 @@ int main(int argc, char **argv)
 	}
 	else
 	{
-		STARPU_LU(lu_decomposition)(A, size, size, nblocks);
+		ret = STARPU_LU(lu_decomposition)(A, size, size, nblocks);
 	}
 
 	if (profile)
 	{
+		FPRINTF(stderr, "Setting profile\n");
 		starpu_profiling_status_set(STARPU_PROFILING_DISABLE);
 		starpu_bus_profiling_helper_display_summary();
 	}
 
-	if (bound) {
+	if (bound)
+	{
 		double min;
+		FPRINTF(stderr, "Setting bound\n");
 		starpu_bound_stop();
-		if (bounddeps) {
+		if (bounddeps)
+		{
 			FILE *f = fopen("lu.pl", "w");
 			starpu_bound_print_lp(f);
 			FPRINTF(stderr,"system printed to lu.pl\n");
-		} else {
+			fclose(f);
+			f = fopen("lu.mps", "w");
+			starpu_bound_print_mps(f);
+			FPRINTF(stderr,"system printed to lu.mps\n");
+			fclose(f);
+			f = fopen("lu.dot", "w");
+			starpu_bound_print_dot(f);
+			FPRINTF(stderr,"system printed to lu.mps\n");
+			fclose(f);
+		}
+		else
+		{
 			starpu_bound_compute(&min, NULL, 0);
 			if (min != 0.)
-				FPRINTF(stderr, "theoretical min: %lf ms\n", min);
+				FPRINTF(stderr, "theoretical min: %f ms\n", min);
 		}
 	}
 
 	if (check)
 	{
-		if (pivot)
+		FPRINTF(stderr, "Checking result\n");
+		if (pivot) {
 			pivot_saved_matrix(ipiv);
+			free(ipiv);
+		}
 
 		check_result();
 	}
 
+	starpu_free(A);
+
+	FPRINTF(stderr, "Shutting down\n");
 	starpu_helper_cublas_shutdown();
 
 	starpu_shutdown();
 
-	return 0;
+	if (ret == -ENODEV) return 77; else return 0;
 }
