@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Centre National de la Recherche Scientifique
- * Copyright (C) 2010  Université de Bordeaux 1
+ * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2011  Université de Bordeaux 1
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -36,6 +36,10 @@ static unsigned sizez = 64*SIZE;
 /* Number of blocks (scattered over the different MPI processes) */
 unsigned nbz = 64;
 
+/* StarPU top variables */
+struct starpu_top_data* starpu_top_init_loop;
+struct starpu_top_data* starpu_top_achieved_loop;
+
 /*
  *	Initialization
  */
@@ -63,36 +67,45 @@ unsigned get_ticks(void)
 static void parse_args(int argc, char **argv)
 {
 	int i;
-	for (i = 1; i < argc; i++) {
-		if (strcmp(argv[i], "-b") == 0) {
+	for (i = 1; i < argc; i++)
+	{
+		if (strcmp(argv[i], "-b") == 0)
+		{
 			bind_tasks = 1;
 		}
 
-		if (strcmp(argv[i], "-nbz") == 0) {
+		if (strcmp(argv[i], "-nbz") == 0)
+		{
 			nbz = atoi(argv[++i]);
 		}
 
-		if (strcmp(argv[i], "-sizex") == 0) {
+		if (strcmp(argv[i], "-sizex") == 0)
+		{
 			sizex = atoi(argv[++i]);
 		}
 
-		if (strcmp(argv[i], "-sizey") == 0) {
+		if (strcmp(argv[i], "-sizey") == 0)
+		{
 			sizey = atoi(argv[++i]);
 		}
 
-		if (strcmp(argv[i], "-sizez") == 0) {
+		if (strcmp(argv[i], "-sizez") == 0)
+		{
 			sizez = atoi(argv[++i]);
 		}
 
-		if (strcmp(argv[i], "-niter") == 0) {
+		if (strcmp(argv[i], "-niter") == 0)
+		{
 			niter = atoi(argv[++i]);
 		}
 
-		if (strcmp(argv[i], "-ticks") == 0) {
+		if (strcmp(argv[i], "-ticks") == 0)
+		{
 			ticks = atoi(argv[++i]);
 		}
 
-		if (strcmp(argv[i], "-h") == 0) {
+		if (strcmp(argv[i], "-h") == 0)
+		{
 			 fprintf(stderr, "Usage : %s [options...]\n", argv[0]);
 			 fprintf(stderr, "\n");
 			 fprintf(stderr, "Options:\n");
@@ -109,6 +122,13 @@ static void parse_args(int argc, char **argv)
 static void init_problem(int argc, char **argv, int rank, int world_size)
 {
 	parse_args(argc, argv);
+
+	if (getenv("STARPU_TOP"))
+	{
+		starpu_top_init_loop = starpu_top_add_data_integer("Task creation iter", 0, niter, 1);
+		starpu_top_achieved_loop = starpu_top_add_data_integer("Task achieved iter", 0, niter, 1);
+		starpu_top_init_and_wait("stencil_top example");
+	}
 	create_blocks_array(sizex, sizey, sizez, nbz);
 
 	/* Select the MPI process which should compute the different blocks */
@@ -122,9 +142,9 @@ static void init_problem(int argc, char **argv, int rank, int world_size)
 	display_memory_consumption(rank);
 
 	who_runs_what_len = 2*niter;
-	who_runs_what = calloc(nbz * who_runs_what_len, sizeof(*who_runs_what));
-	who_runs_what_index = calloc(nbz, sizeof(*who_runs_what_index));
-	last_tick = calloc(nbz, sizeof(*last_tick));
+	who_runs_what = (int *) calloc(nbz * who_runs_what_len, sizeof(*who_runs_what));
+	who_runs_what_index = (int *) calloc(nbz, sizeof(*who_runs_what_index));
+	last_tick = (struct timeval *) calloc(nbz, sizeof(*last_tick));
 }
 
 /*
@@ -142,8 +162,10 @@ void f(unsigned task_per_worker[STARPU_NMAXWORKERS])
 
 	for (worker = 0; worker < STARPU_NMAXWORKERS; worker++)
 		total += task_per_worker[worker];
-	for (worker = 0; worker < STARPU_NMAXWORKERS; worker++) {
-		if (task_per_worker[worker]) {
+	for (worker = 0; worker < STARPU_NMAXWORKERS; worker++)
+	{
+		if (task_per_worker[worker])
+		{
 			char name[32];
 			starpu_worker_get_name(worker, name, sizeof(name));
 			fprintf(stderr,"\t%s -> %d (%2.2f%%)\n", name, task_per_worker[worker], (100.0*task_per_worker[worker])/total);
@@ -168,10 +190,12 @@ int main(int argc, char **argv)
 {
 	int rank;
 	int world_size;
+	int ret;
 
 #ifdef STARPU_USE_MPI
 	int thread_support;
-	if (MPI_Init_thread(&argc, &argv, MPI_THREAD_SERIALIZED, &thread_support)) {
+	if (MPI_Init_thread(&argc, &argv, MPI_THREAD_SERIALIZED, &thread_support))
+	{
 		fprintf(stderr, "MPI_Init_thread failed\n");
 	}
 	if (thread_support == MPI_THREAD_FUNNELED)
@@ -191,7 +215,10 @@ int main(int argc, char **argv)
 		fflush(stderr);
 	}
 
-	starpu_init(NULL);
+	ret = starpu_init(NULL);
+	if (ret == -ENODEV)
+		return 77;
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 #ifdef STARPU_USE_MPI
 	starpu_mpi_initialize();
@@ -285,15 +312,18 @@ int main(int argc, char **argv)
 
 		unsigned bz, iter;
 		unsigned last;
-		for (iter = 0; iter < who_runs_what_len; iter++) {
+		for (iter = 0; iter < who_runs_what_len; iter++)
+		{
 			last = 1;
-			for (bz = 0; bz < nbz; bz++) {
+			for (bz = 0; bz < nbz; bz++)
+			{
 				if ((bz % nzblocks_per_process) == 0)
 					fprintf(stderr, "| ");
 
 				if (who_runs_what_index[bz] <= iter)
 					fprintf(stderr,"_ ");
-				else {
+				else
+				{
 					last = 0;
 					if (who_runs_what[bz + iter * nbz] == -1)
 						fprintf(stderr,"* ");
@@ -310,10 +340,10 @@ int main(int argc, char **argv)
 
 		fflush(stderr);
 
-		fprintf(stdout, "Computation took: %lf ms on %d MPI processes\n", max_timing/1000, world_size);
-		fprintf(stdout, "\tMIN : %lf ms\n", min_timing/1000);
-		fprintf(stdout, "\tMAX : %lf ms\n", max_timing/1000);
-		fprintf(stdout, "\tAVG : %lf ms\n", sum_timing/(world_size*1000));
+		fprintf(stdout, "Computation took: %f ms on %d MPI processes\n", max_timing/1000, world_size);
+		fprintf(stdout, "\tMIN : %f ms\n", min_timing/1000);
+		fprintf(stdout, "\tMAX : %f ms\n", max_timing/1000);
+		fprintf(stdout, "\tAVG : %f ms\n", sum_timing/(world_size*1000));
 	}
 
 	starpu_shutdown();
