@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010  Université de Bordeaux 1
- * Copyright (C) 2010, 2011  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009, 2010, 2012  Université de Bordeaux 1
+ * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,6 +15,15 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+/* This example shows how to submit a series of tasks in a chain of dependency:
+ *
+ * ... -> task (i) --> task (i+1) --> ...
+ *
+ * but here submitted in reverse order.
+ *
+ * This is repeated several times
+ */
+
 #include <string.h>
 #include <math.h>
 #include <sys/types.h>
@@ -30,7 +39,7 @@
 #define FPRINTF(ofile, fmt, args ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ##args); }} while(0)
 #define TAG(i, iter)	((starpu_tag_t)  (((uint64_t)iter)<<32 | (i)) )
 
-starpu_codelet cl;
+struct starpu_codelet cl = {};
 
 #define Ni	64
 #define Nk	256
@@ -40,18 +49,22 @@ static unsigned ni = Ni, nk = Nk;
 static void parse_args(int argc, char **argv)
 {
 	int i;
-	for (i = 1; i < argc; i++) {
-		if (strcmp(argv[i], "-iter") == 0) {
+	for (i = 1; i < argc; i++)
+	{
+		if (strcmp(argv[i], "-iter") == 0)
+		{
 		        char *argptr;
 			nk = strtol(argv[++i], &argptr, 10);
 		}
 
-		if (strcmp(argv[i], "-i") == 0) {
+		if (strcmp(argv[i], "-i") == 0)
+		{
 		        char *argptr;
 			ni = strtol(argv[++i], &argptr, 10);
 		}
 
-		if (strcmp(argv[i], "-h") == 0) {
+		if (strcmp(argv[i], "-h") == 0)
+		{
 			printf("usage : %s [-iter iter] [-i i]\n", argv[0]);
 		}
 	}
@@ -64,16 +77,13 @@ static void tag_cleanup_grid(unsigned ni, unsigned iter)
 	unsigned i;
 
 	for (i = 0; i < ni; i++)
-	{
 		starpu_tag_remove(TAG(i,iter));
-	}
-
-
 } 
 
-static void create_task_grid(unsigned iter)
+static int create_task_grid(unsigned iter)
 {
 	int i;
+	int ret;
 
 /*	FPRINTF(stderr, "start iter %d ni %d...\n", iter, ni); */
 
@@ -91,20 +101,31 @@ static void create_task_grid(unsigned iter)
 		if (i != 1)
 			starpu_tag_declare_deps(TAG(i,iter), 1, TAG(i-1,iter));
 
-		starpu_task_submit(task);
+		ret = starpu_task_submit(task);
+		if (ret == -ENODEV) return 77;
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 	}
-
+	return 0;
 }
 
-void cpu_codelet(void *descr[], void *_args __attribute__ ((unused)))
+void cpu_codelet(void *descr[] __attribute__ ((unused)), void *_args __attribute__ ((unused)))
 {
 }
 
 int main(int argc __attribute__((unused)) , char **argv __attribute__((unused)))
 {
 	unsigned i;
+	int ret;
 
-	starpu_init(NULL);
+	ret = starpu_init(NULL);
+	if (ret == -ENODEV)
+		exit(77);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
+
+#ifdef STARPU_SLOW_MACHINE
+	ni /= 4;
+	nk /= 16;
+#endif
 
 #ifdef STARPU_USE_GORDON
 	/* load an empty kernel and get its identifier */
@@ -113,19 +134,20 @@ int main(int argc __attribute__((unused)) , char **argv __attribute__((unused)))
 
 	parse_args(argc, argv);
 
-	cl.cpu_func = cpu_codelet;
-	cl.cuda_func = cpu_codelet;
+	cl.cpu_funcs[0] = cpu_codelet;
+	cl.cuda_funcs[0] = cpu_codelet;
+	cl.opencl_funcs[0] = cpu_codelet;
 #ifdef STARPU_USE_GORDON
 	cl.gordon_func = gordon_null_kernel;
 #endif
-	cl.where = STARPU_CPU|STARPU_CUDA|STARPU_GORDON;
 	cl.nbuffers = 0;
 
 	FPRINTF(stderr, "ITER : %u\n", nk);
 
 	for (i = 0; i < nk; i++)
 	{
-		create_task_grid(i);
+		ret = create_task_grid(i);
+		if (ret == 77) goto enodev;
 
 		starpu_tag_wait(TAG(ni-1, i));
 
@@ -134,9 +156,10 @@ int main(int argc __attribute__((unused)) , char **argv __attribute__((unused)))
 			tag_cleanup_grid(ni, i-1);
 	}
 
+enodev:
 	starpu_shutdown();
 
 	FPRINTF(stderr, "TEST DONE ...\n");
 
-	return 0;
+	return ret;
 }
