@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2010, 2011  Université de Bordeaux 1
- * Copyright (C) 2010  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2012  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -90,7 +90,7 @@ static struct starpu_task *create_task(starpu_tag_t id)
 
 /* Send handle to every node appearing in the mask, and unlock tag once the
  * transfers are done. */
-static void send_data_to_mask(starpu_data_handle handle, int *rank_mask, int mpi_tag, starpu_tag_t tag)
+static void send_data_to_mask(starpu_data_handle_t handle, int *rank_mask, int mpi_tag, starpu_tag_t tag)
 {
 	unsigned cnt = 0;
 
@@ -99,7 +99,7 @@ static void send_data_to_mask(starpu_data_handle handle, int *rank_mask, int mpi
 	int rank_array[world_size];
 	MPI_Comm comm_array[world_size];
 	int mpi_tag_array[world_size];
-	starpu_data_handle handle_array[world_size];
+	starpu_data_handle_t handle_array[world_size];
 
 	unsigned r;
 	for (r = 0; r < world_size; r++)
@@ -132,7 +132,7 @@ static void send_data_to_mask(starpu_data_handle handle, int *rank_mask, int mpi
 struct recv_when_done_callback_arg {
 	int source;
 	int mpi_tag;
-	starpu_data_handle handle;
+	starpu_data_handle_t handle;
 	starpu_tag_t unlocked_tag;
 };
 
@@ -148,7 +148,7 @@ static void callback_receive_when_done(void *_arg)
 
 static void receive_when_deps_are_done(unsigned ndeps, starpu_tag_t *deps_tags,
 				int source, int mpi_tag,
-				starpu_data_handle handle,
+				starpu_data_handle_t handle,
 				starpu_tag_t partial_tag,
 				starpu_tag_t unlocked_tag)
 {
@@ -206,9 +206,9 @@ static void create_task_11_recv(unsigned k)
 	
 	int source = get_block_rank(k, k);
 #ifdef SINGLE_TMP11
-	starpu_data_handle block_handle = STARPU_PLU(get_tmp_11_block_handle)();
+	starpu_data_handle_t block_handle = STARPU_PLU(get_tmp_11_block_handle)();
 #else
-	starpu_data_handle block_handle = STARPU_PLU(get_tmp_11_block_handle)(k);
+	starpu_data_handle_t block_handle = STARPU_PLU(get_tmp_11_block_handle)(k);
 #endif
 	int mpi_tag = MPI_TAG11(k);
 	starpu_tag_t partial_tag = TAG11_SAVE_PARTIAL(k);
@@ -250,7 +250,7 @@ static void callback_task_11_real(void *_arg)
 	rank_mask[rank] = 0;
 
 	/* Send the block to those nodes */
-	starpu_data_handle block_handle = STARPU_PLU(get_block_handle)(k, k);
+	starpu_data_handle_t block_handle = STARPU_PLU(get_block_handle)(k, k);
 	starpu_tag_t tag = TAG11_SAVE(k);
 	int mpi_tag = MPI_TAG11(k);
 	send_data_to_mask(block_handle, rank_mask, mpi_tag, tag);
@@ -267,8 +267,7 @@ static void create_task_11_real(unsigned k)
 	task->cl_arg = create_debug_info(k, k, k);
 
 	/* which sub-data is manipulated ? */
-	task->buffers[0].handle = STARPU_PLU(get_block_handle)(k, k);
-	task->buffers[0].mode = STARPU_RW;
+	task->handles[0] = STARPU_PLU(get_block_handle)(k, k);
 
 	struct callback_arg *arg = malloc(sizeof(struct callback_arg));
 		arg->k = k;
@@ -288,7 +287,8 @@ static void create_task_11_real(unsigned k)
 		starpu_tag_declare_deps(TAG11(k), 1, STARPU_TAG_INIT);
 	}
 
-	starpu_task_submit(task);
+	int ret = starpu_task_submit(task);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 }
 
 static void create_task_11(unsigned k)
@@ -357,9 +357,9 @@ static void create_task_12_recv(unsigned k, unsigned j)
 	
 	int source = get_block_rank(k, j);
 #ifdef SINGLE_TMP1221
-	starpu_data_handle block_handle = STARPU_PLU(get_tmp_12_block_handle)(j);
+	starpu_data_handle_t block_handle = STARPU_PLU(get_tmp_12_block_handle)(j);
 #else
-	starpu_data_handle block_handle = STARPU_PLU(get_tmp_12_block_handle)(j,k);
+	starpu_data_handle_t block_handle = STARPU_PLU(get_tmp_12_block_handle)(j,k);
 #endif
 	int mpi_tag = MPI_TAG12(k, j);
 	starpu_tag_t partial_tag = TAG12_SAVE_PARTIAL(k, j);
@@ -394,7 +394,7 @@ static void callback_task_12_real(void *_arg)
 	rank_mask[rank] = 0;
 
 	/* Send the block to those nodes */
-	starpu_data_handle block_handle = STARPU_PLU(get_block_handle)(k, j);
+	starpu_data_handle_t block_handle = STARPU_PLU(get_block_handle)(k, j);
 	starpu_tag_t tag = TAG12_SAVE(k, j);
 	int mpi_tag = MPI_TAG12(k, j);
 	send_data_to_mask(block_handle, rank_mask, mpi_tag, tag);
@@ -417,7 +417,7 @@ static void create_task_12_real(unsigned k, unsigned j)
 	starpu_tag_t tag_11_dep; 
 
 	/* which sub-data is manipulated ? */
-	starpu_data_handle diag_block;
+	starpu_data_handle_t diag_block;
 	if (diag_block_is_local)
 	{
 		diag_block = STARPU_PLU(get_block_handle)(k, k);
@@ -433,15 +433,13 @@ static void create_task_12_real(unsigned k, unsigned j)
 		tag_11_dep = TAG11_SAVE(k);
 	}
 
-	task->buffers[0].handle = diag_block; 
-	task->buffers[0].mode = STARPU_R;
-	task->buffers[1].handle = STARPU_PLU(get_block_handle)(k, j); 
-	task->buffers[1].mode = STARPU_RW;
+	task->handles[0] = diag_block; 
+	task->handles[1] = STARPU_PLU(get_block_handle)(k, j); 
 
 	STARPU_ASSERT(get_block_rank(k, j) == rank);
 
-	STARPU_ASSERT(task->buffers[0].handle != STARPU_POISON_PTR);
-	STARPU_ASSERT(task->buffers[1].handle != STARPU_POISON_PTR);
+	STARPU_ASSERT(task->handles[0] != STARPU_POISON_PTR);
+	STARPU_ASSERT(task->handles[1] != STARPU_POISON_PTR);
 
 	struct callback_arg *arg = malloc(sizeof(struct callback_arg));
 		arg->j = j;
@@ -462,7 +460,8 @@ static void create_task_12_real(unsigned k, unsigned j)
 		starpu_tag_declare_deps(TAG12(k, j), 1, tag_11_dep);
 	}
 
-	starpu_task_submit(task);
+	int ret = starpu_task_submit(task);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 }
 
 static void create_task_12(unsigned k, unsigned j)
@@ -529,9 +528,9 @@ static void create_task_21_recv(unsigned k, unsigned i)
 
 	int source = get_block_rank(i, k);
 #ifdef SINGLE_TMP1221
-	starpu_data_handle block_handle = STARPU_PLU(get_tmp_21_block_handle)(i);
+	starpu_data_handle_t block_handle = STARPU_PLU(get_tmp_21_block_handle)(i);
 #else
-	starpu_data_handle block_handle = STARPU_PLU(get_tmp_21_block_handle)(i, k);
+	starpu_data_handle_t block_handle = STARPU_PLU(get_tmp_21_block_handle)(i, k);
 #endif
 	int mpi_tag = MPI_TAG21(k, i);
 	starpu_tag_t partial_tag = TAG21_SAVE_PARTIAL(k, i);
@@ -567,7 +566,7 @@ static void callback_task_21_real(void *_arg)
 	rank_mask[rank] = 0;
 
 	/* Send the block to those nodes */
-	starpu_data_handle block_handle = STARPU_PLU(get_block_handle)(i, k);
+	starpu_data_handle_t block_handle = STARPU_PLU(get_block_handle)(i, k);
 	starpu_tag_t tag = TAG21_SAVE(k, i);
 	int mpi_tag = MPI_TAG21(k, i);
 	send_data_to_mask(block_handle, rank_mask, mpi_tag, tag);
@@ -590,7 +589,7 @@ static void create_task_21_real(unsigned k, unsigned i)
 	starpu_tag_t tag_11_dep; 
 	
 	/* which sub-data is manipulated ? */
-	starpu_data_handle diag_block;
+	starpu_data_handle_t diag_block;
 	if (diag_block_is_local)
 	{
 		diag_block = STARPU_PLU(get_block_handle)(k, k);
@@ -606,13 +605,11 @@ static void create_task_21_real(unsigned k, unsigned i)
 		tag_11_dep = TAG11_SAVE(k);
 	}
 
-	task->buffers[0].handle = diag_block; 
-	task->buffers[0].mode = STARPU_R;
-	task->buffers[1].handle = STARPU_PLU(get_block_handle)(i, k);
-	task->buffers[1].mode = STARPU_RW;
+	task->handles[0] = diag_block; 
+	task->handles[1] = STARPU_PLU(get_block_handle)(i, k);
 
-	STARPU_ASSERT(task->buffers[0].handle != STARPU_POISON_PTR);
-	STARPU_ASSERT(task->buffers[1].handle != STARPU_POISON_PTR);
+	STARPU_ASSERT(task->handles[0] != STARPU_POISON_PTR);
+	STARPU_ASSERT(task->handles[1] != STARPU_POISON_PTR);
 
 	struct callback_arg *arg = malloc(sizeof(struct callback_arg));
 		arg->i = i;
@@ -633,7 +630,8 @@ static void create_task_21_real(unsigned k, unsigned i)
 		starpu_tag_declare_deps(TAG21(k, i), 1, tag_11_dep);
 	}
 
-	starpu_task_submit(task);
+	int ret = starpu_task_submit(task);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 }
 
 static void create_task_21(unsigned k, unsigned i)
@@ -685,7 +683,7 @@ static void create_task_22_real(unsigned k, unsigned i, unsigned j)
 	unsigned block21_is_local = (get_block_rank(i, k) == rank);
 	starpu_tag_t tag_21_dep;
 
-	starpu_data_handle block21;
+	starpu_data_handle_t block21;
 	if (block21_is_local)
 	{
 		block21 = STARPU_PLU(get_block_handle)(i, k);
@@ -705,7 +703,7 @@ static void create_task_22_real(unsigned k, unsigned i, unsigned j)
 	unsigned block12_is_local = (get_block_rank(k, j) == rank);
 	starpu_tag_t tag_12_dep;
 
-	starpu_data_handle block12;
+	starpu_data_handle_t block12;
 	if (block12_is_local)
 	{
 	//	block12 = STARPU_PLU(get_block_handle)(j, k);
@@ -725,21 +723,18 @@ static void create_task_22_real(unsigned k, unsigned i, unsigned j)
 
 
 #warning temporary fix :/
-	//task->buffers[0].handle = block21;
-	task->buffers[0].handle = block12;
-	task->buffers[0].mode = STARPU_R;
+	//task->handles[0] = block21;
+	task->handles[0] = block12;
 
-	//task->buffers[1].handle = block12;
-	task->buffers[1].handle = block21;
-	task->buffers[1].mode = STARPU_R;
+	//task->handles[1] = block12;
+	task->handles[1] = block21;
 
 	/* produced by TAG22(k-1, i, j) */
-	task->buffers[2].handle = STARPU_PLU(get_block_handle)(i, j);
-	task->buffers[2].mode = STARPU_RW;
+	task->handles[2] = STARPU_PLU(get_block_handle)(i, j);
 
-	STARPU_ASSERT(task->buffers[0].handle != STARPU_POISON_PTR);
-	STARPU_ASSERT(task->buffers[1].handle != STARPU_POISON_PTR);
-	STARPU_ASSERT(task->buffers[2].handle != STARPU_POISON_PTR);
+	STARPU_ASSERT(task->handles[0] != STARPU_POISON_PTR);
+	STARPU_ASSERT(task->handles[1] != STARPU_POISON_PTR);
+	STARPU_ASSERT(task->handles[2] != STARPU_POISON_PTR);
 
 	if (!no_prio &&  (i == k + 1) && (j == k +1) ) {
 		task->priority = STARPU_MAX_PRIO;
@@ -753,7 +748,8 @@ static void create_task_22_real(unsigned k, unsigned i, unsigned j)
 		starpu_tag_declare_deps(TAG22(k, i, j), 2, tag_12_dep, tag_21_dep);
 	}
 
-	starpu_task_submit(task);
+	int ret = starpu_task_submit(task);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 }
 
 static void create_task_22(unsigned k, unsigned i, unsigned j)
@@ -768,7 +764,7 @@ static void create_task_22(unsigned k, unsigned i, unsigned j)
 //	}
 }
 
-static void wait_tag_and_fetch_handle(starpu_tag_t tag, starpu_data_handle handle)
+static void wait_tag_and_fetch_handle(starpu_tag_t tag, starpu_data_handle_t handle)
 {
 	STARPU_ASSERT(handle != STARPU_POISON_PTR);
 
@@ -788,7 +784,7 @@ static void wait_termination(void)
 		/* Wait task 11k if needed */
 		if (get_block_rank(k, k) == rank)
 		{
-			starpu_data_handle diag_block = STARPU_PLU(get_block_handle)(k, k);
+			starpu_data_handle_t diag_block = STARPU_PLU(get_block_handle)(k, k);
 			wait_tag_and_fetch_handle(TAG11_SAVE(k), diag_block);
 		}
 		
@@ -798,8 +794,8 @@ static void wait_termination(void)
 			/* Wait task 21ki if needed */
 			if (get_block_rank(i, k) == rank)
 			{
-				starpu_data_handle block21 = STARPU_PLU(get_block_handle)(i, k);
-				//starpu_data_handle block21 = STARPU_PLU(get_block_handle)(k, i);
+				starpu_data_handle_t block21 = STARPU_PLU(get_block_handle)(i, k);
+				//starpu_data_handle_t block21 = STARPU_PLU(get_block_handle)(k, i);
 				//fprintf(stderr, "BLOCK21 i %d k %d -> handle %p\n", i, k, block21);
 				wait_tag_and_fetch_handle(TAG21_SAVE(k, i), block21);
 			}
@@ -810,8 +806,8 @@ static void wait_termination(void)
 			/* Wait task 12kj if needed */
 			if (get_block_rank(k, j) == rank)
 			{
-				//starpu_data_handle block12 = STARPU_PLU(get_block_handle)(j, k);
-				starpu_data_handle block12 = STARPU_PLU(get_block_handle)(k, j);
+				//starpu_data_handle_t block12 = STARPU_PLU(get_block_handle)(j, k);
+				starpu_data_handle_t block12 = STARPU_PLU(get_block_handle)(k, j);
 				//fprintf(stderr, "BLOCK12 j %d k %d -> handle %p\n", j, k, block12);
 				wait_tag_and_fetch_handle(TAG12_SAVE(k, j), block12);
 			}
@@ -868,7 +864,7 @@ double STARPU_PLU(plu_main)(unsigned _nblocks, int _rank, int _world_size)
 
 	double timing = (double)((end.tv_sec - start.tv_sec)*1000000 + (end.tv_usec - start.tv_usec));
 	
-//	fprintf(stderr, "RANK %d -> took %lf ms\n", rank, timing/1000);
+//	fprintf(stderr, "RANK %d -> took %f ms\n", rank, timing/1000);
 	
 	return timing;
 }
