@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
+ * Copyright (C) 2010, 2012  Université de Bordeaux 1
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,19 +22,24 @@
 
 #include <starpu_scheduler.h>
 
-struct starpu_machine_config_s;
-struct starpu_sched_policy_s *_starpu_get_sched_policy(void);
+struct _starpu_machine_config;
+struct starpu_sched_policy *_starpu_get_sched_policy(void);
 
-void _starpu_init_sched_policy(struct starpu_machine_config_s *config);
-void _starpu_deinit_sched_policy(struct starpu_machine_config_s *config);
+void _starpu_init_sched_policy(struct _starpu_machine_config *config);
+void _starpu_deinit_sched_policy(struct _starpu_machine_config *config);
 
-int _starpu_push_task(starpu_job_t task, unsigned job_is_already_locked);
+int _starpu_push_task(struct _starpu_job *task);
 /* pop a task that can be executed on the worker */
-struct starpu_task *_starpu_pop_task(struct starpu_worker_s *worker);
+struct starpu_task *_starpu_pop_task(struct _starpu_worker *worker);
 /* pop every task that can be executed on the worker */
 struct starpu_task *_starpu_pop_every_task(void);
 void _starpu_sched_post_exec_hook(struct starpu_task *task);
 
 void _starpu_wait_on_sched_event(void);
 
+struct starpu_task *_starpu_create_conversion_task(starpu_data_handle_t handle,
+						   unsigned int node);
+
+void _starpu_sched_pre_exec_hook(struct starpu_task *task);
+
 #endif // __SCHED_POLICY_H__
