@@ -25,15 +25,15 @@
 extern int _starpu_bound_recording;
 
 /* Record task for bound computation */
-extern void _starpu_bound_record(starpu_job_t j);
+extern void _starpu_bound_record(struct _starpu_job *j);
 
 /* Record tag dependency: id depends on dep_id */
 extern void _starpu_bound_tag_dep(starpu_tag_t id, starpu_tag_t dep_id);
 
 /* Record task dependency: j depends on dep_j */
-extern void _starpu_bound_task_dep(starpu_job_t j, starpu_job_t dep_j);
+extern void _starpu_bound_task_dep(struct _starpu_job *j, struct _starpu_job *dep_j);
 
 /* Record job id dependency: j depends on job_id */
-extern void _starpu_bound_job_id_dep(starpu_job_t dep_j, unsigned long job_id);
+extern void _starpu_bound_job_id_dep(struct _starpu_job *dep_j, unsigned long job_id);
 
 #endif // __BOUND_H__
